package meerkat.bulletinboard;

import com.google.protobuf.ByteString;
import com.google.protobuf.Message;
import meerkat.crypto.DigitalSignature;
import meerkat.protobuf.BulletinBoardAPI.BatchData;
import meerkat.protobuf.Crypto;

import java.io.IOException;
import java.io.InputStream;
import java.security.InvalidKeyException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.SignatureException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;

/**
 * Created by Arbel Deutsch Peled on 20-Dec-15.
 * Wrapper class for signing and verifying Batch signatures in a standardized way
 */
public class GenericBatchDigitalSignature implements BatchDigitalSignature{

    private DigitalSignature digitalSignature;

    public GenericBatchDigitalSignature(DigitalSignature digitalSignature) {
        this.digitalSignature = digitalSignature;
    }

    @Override
    public void updateContent(CompleteBatch completeBatch) throws SignatureException {

        digitalSignature.updateContent(completeBatch.getBeginBatchMessage());
        for (BatchData batchData : completeBatch.getBatchDataList()) {
            digitalSignature.updateContent(batchData);
        }

    }

    @Override
    public boolean verify(CompleteBatch completeBatch) throws SignatureException, CertificateException, InvalidKeyException {

        digitalSignature.initVerify(completeBatch.getSignature());
        updateContent(completeBatch);
        return digitalSignature.verify();

    }

    @Override
    public void loadVerificationCertificates(InputStream certStream) throws CertificateException {
        digitalSignature.loadVerificationCertificates(certStream);
    }

    @Override
    public void clearVerificationCertificates() {
        digitalSignature.clearVerificationCertificates();
    }

    @Override
    public void updateContent(Message msg) throws SignatureException {
        digitalSignature.updateContent(msg);
    }

    @Override
    public Crypto.Signature sign() throws SignatureException {
        return digitalSignature.sign();
    }

    @Override
    public void initVerify(Crypto.Signature sig) throws CertificateException, InvalidKeyException {
        digitalSignature.initVerify(sig);
    }

    @Override
    public boolean verify() {
        return digitalSignature.verify();
    }

    @Override
    public KeyStore.Builder getPKCS12KeyStoreBuilder(InputStream keyStream, char[] password)
            throws IOException, CertificateException, KeyStoreException, NoSuchAlgorithmException {
        return digitalSignature.getPKCS12KeyStoreBuilder(keyStream, password);
    }

    @Override
    public void loadSigningCertificate(KeyStore.Builder keyStoreBuilder) throws IOException, CertificateException, UnrecoverableKeyException {
        digitalSignature.loadSigningCertificate(keyStoreBuilder);
    }

    @Override
    public ByteString getSignerID() {
        return digitalSignature.getSignerID();
    }

    @Override
    public void clearSigningKey() {
        digitalSignature.clearSigningKey();
    }

}
